@@ -17,18 +17,20 @@
 import logging
 import re
 from collections import OrderedDict
-from datetime import datetime, timedelta
-from typing import Dict, List
+from datetime import datetime, timedelta, timezone
+from typing import Dict, List, Union
 
 import ee
 import tabulate
 from tabulate import TableFormat, Line, DataRow
 
-from geedim import info, medoid
-from geedim.download import BaseImage, split_id
+from geedim import schema, medoid
+from geedim.download import BaseImage
 from geedim.enums import ResamplingMethod, CompositeMethod
-from geedim.errors import UnfilteredError, ComponentImageError
+from geedim.errors import UnfilteredError, InputImageError
 from geedim.mask import MaskedImage, class_from_id
+from geedim.stac import StacCatalog, StacItem
+from geedim.utils import split_id, resample
 
 logger = logging.getLogger(__name__)
 tabulate.MIN_PADDING = 0
@@ -43,8 +45,8 @@ _table_fmt = TableFormat(
     headerrow=DataRow("", " ", ""),
     datarow=DataRow("", " ", ""),
     padding=0,
-    with_header_hide=["lineabove", "linebelow"],
-)
+    with_header_hide=["lineabove", "linebelow"]
+) # yapf: disable
 
 
 def compatible_collections(names: List[str]) -> bool:
@@ -68,45 +70,53 @@ def compatible_collections(names: List[str]) -> bool:
         name_matches.append(name_match)
     return all(name_matches)
 
+def parse_date(date: Union[datetime, str], var_name=None) -> datetime:
+    """ Convert a string to a datetime, raising an exception if it is in the wrong format. """
+    var_name = var_name or 'date'
+    if isinstance(date, str):
+        try:
+            date = datetime.strptime(date, '%Y-%m-%d')
+        except ValueError:
+            raise ValueError(f'{var_name} should be a datetime instance or a string with format: "%Y-%m-%d"')
+    return date
 
 class MaskedCollection:
-    """
-    Class for encapsulating, searching and compositing an Earth Engine image collection, with support for
-    cloud/shadow masking.
-    """
 
-    def __init__(self, ee_collection):
+    def __init__(self, ee_collection: ee.ImageCollection):
         """
-        Create a MaskedCollection instance.
+        A class for describing, searching and compositing an Earth Engine image collection, with support for
+        cloud/shadow masking.
 
         Parameters
         ----------
         ee_collection : ee.ImageCollection
-            The Earth Engine image collection to encapsulate.
+            Earth Engine image collection to encapsulate.
         """
         if not isinstance(ee_collection, ee.ImageCollection):
             raise TypeError(f'`ee_collection` must be an instance of ee.ImageCollection')
         self._name = None
-        self._info = None
         self._properties = None
+        self._schema = None
         self._filtered = False
         self._ee_collection = ee_collection
         self._image_type = None
+        self.__stac = None
+        self.__stats_scale = None
 
     @classmethod
-    def from_name(cls, name):
+    def from_name(cls, name: str) -> 'MaskedCollection':
         """
         Create a MaskedCollection instance from an Earth Engine image collection name.
 
         Parameters
         ----------
         name: str
-            The name of the Earth Engine image collection to create.
+            Name of the Earth Engine image collection to create.
 
         Returns
         -------
-        gd_collection: MaskedCollection
-            The MaskedCollection instance.
+        MaskedCollection
+            A MaskedCollection instance.
         """
         # this is separate from __init__ for consistency with MaskedImage.from_id()
         if (name == 'COPERNICUS/S2') or (name == 'COPERNICUS/S2_SR'):
@@ -124,25 +134,31 @@ class MaskedCollection:
         return gd_collection
 
     @classmethod
-    def from_list(cls, image_list):
+    def from_list(cls, image_list: List[Union[str, MaskedImage, ee.Image]]) -> 'MaskedCollection':
         """
-        Create a MaskedCollection instance from a list of EE image ID strings, ee.Image objects and/or MaskedImage
-        objects.
+        Create a MaskedCollection instance from a list of Earth Engine image ID strings, ``ee.Image`` instances and/or
+        :class:`~geedim.mask.MaskedImage` instances.  The list may include composite images, as created with
+        :meth:`composite`.
 
-        Any ee.Image objects (including those encapsulated by MaskedImage) must have `system:id` and
-        `system:time_start` properties.  This is always the case for images from the EE data catalog, and images
-        returned from MaskedCollection.composite(), but user-created images passed to from_list() should have these
-        properties set.
+        Images from spectrally compatible Landsat collections can be combined i.e. Landsat-4 with Landsat-5,
+        and Landsat-8 with Landsat-9.  Otherwise, images should all belong to the same collection.
+
+        Any ``ee.Image`` instances in the list (including those encapsulated by :class:`~geedim.mask.MaskedImage`)
+        must have ``system:id`` and ``system:time_start`` properties.  This is always the case for images
+        obtained from the Earth Engine catalog, and images returned from :meth:`composite`.  Any other user-created
+        images passed to :meth:`from_list` should have these properties set.
 
         Parameters
         ----------
         image_list : list
-            A list of images to include in the collection (must all be from the same EE collection).
+            List of images to include in the collection (must all be from the same, or compatible Earth Engine
+            collections). List items can be ID strings, instances of :class:`~geedim.mask.MaskedImage`, or instances of
+            ``ee.Image``.
 
         Returns
         -------
-        gd_collection: MaskedCollection
-            The MaskedCollection instance.
+        MaskedCollection
+            A MaskedCollection instance.
         """
         # build lists of EE images and IDs from image_list
         if len(image_list) == 0:
@@ -166,26 +182,41 @@ class MaskedCollection:
 
         # check all images have IDs and capture dates
         if any([(im_dict['id'] is None) or (not im_dict['has_date']) for im_dict in im_dict_list]):
-            raise ComponentImageError('Image(s) must have "id" and "system:time_start" properties.')
+            raise InputImageError('Image(s) must have "id" and "system:time_start" properties.')
 
         # check the images all come from the same or compatible collections
         ee_coll_names = [split_id(im_dict['id'])[0] for im_dict in im_dict_list]
         if not compatible_collections(ee_coll_names):
-            raise ComponentImageError(
-                'All images must belong to the same, or spectrally compatible, collections.'
-            )
+            raise InputImageError('All images must belong to the same, or spectrally compatible, collections.')
 
         # create the collection object, using the name of the first collection in ee_coll_names.
         gd_collection = cls.from_name(ee_coll_names[0])
-        gd_collection._ee_collection = ee.ImageCollection(
-            ee.List([im_dict['ee_image'] for im_dict in im_dict_list])
-        )
+        gd_collection._ee_collection = ee.ImageCollection(ee.List([im_dict['ee_image'] for im_dict in im_dict_list]))
         gd_collection._filtered = True
         return gd_collection
 
+    @property
+    def _stac(self) -> Union[StacItem, None]:
+        """ STAC info, if any.  """
+        if not self.__stac and (self.name in StacCatalog().url_dict):
+            self.__stac = StacCatalog().get_item(self.name)
+        return self.__stac
+
+    @property
+    def _stats_scale(self) -> Union[float, None]:
+        """ Scale to use for re-projections when finding region statistics. """
+        if not self._stac:
+            return None
+        if not self.__stats_scale:
+            gsds = [float(band_dict['gsd']) for band_dict in self._stac.band_props.values()]
+            max_gsd = max(gsds)
+            min_gsd = min(gsds)
+            self.__stats_scale = min_gsd if (max_gsd > 10 * min_gsd) and (min_gsd > 0) else max_gsd
+        return self.__stats_scale
+
     @property
     def ee_collection(self) -> ee.ImageCollection:
-        """ The encapsulated Earth Engine image collection. """
+        """ Earth Engine image collection. """
         return self._ee_collection
 
     @property
@@ -196,26 +227,19 @@ class MaskedCollection:
             self._name = split_id(ee_info['id'])[0] if ee_info and 'id' in ee_info else 'None'
         return self._name
 
-    @property
-    def info(self) -> Dict:
-        """ Search properties and band metadata. """
-        if not self._info:
-            if self.name in info.collection_info:
-                self._info = info.collection_info[self.name]
-            else:
-                self._info = info.collection_info['*']
-        return self._info
-
     @property
     def image_type(self) -> type:
-        """ geedim class to encapsulate images from `ee_collection`. """
+        """ :class:`~geedim.mask.MaskedImage` class or sub-class corresponding to images in :attr:`ee_collection`. """
         if not self._image_type:
             self._image_type = class_from_id(self.name)
         return self._image_type
 
     @property
-    def properties(self) -> Dict:
-        """ Properties for each image in the collection. """
+    def properties(self) -> Dict[str, Dict]:
+        """
+        A dictionary of properties for each image in the collection. Dictionary keys are the image IDs, and values
+        are a dictionaries of image properties.
+        """
         if not self._filtered:
             raise UnfilteredError(
                 '`properties` can only be retrieved for collections returned by `search()` and `from_list()`'
@@ -226,25 +250,40 @@ class MaskedCollection:
 
     @property
     def properties_table(self) -> str:
-        """ `properties` formatted as a table. """
+        """ :attr:`properties` formatted as a printable table string. """
         return self._get_properties_table(self.properties)
 
     @property
-    def properties_key(self) -> Dict:
-        """ Abbreviations and descriptions for `properties`. """
-        return OrderedDict({key_dict['PROPERTY']: key_dict for key_dict in self.info['properties']})
+    def schema(self) -> Dict[str, Dict]:
+        """
+        A dictionary of abbreviations and descriptions for :attr:`properties`. Keys are the EE property names,
+        and values are dictionaries of abbreviations and descriptions.
+        """
+        if not self._schema:
+            if self.name in schema.collection_schema:
+                self._schema = schema.collection_schema[self.name]['prop_schema']
+            else:
+                self._schema = schema.default_prop_schema
+        return self._schema
 
     @property
-    def key_table(self) -> str:
-        """ `properties_key` formatted as a table. """
-        key_dict = [dict(ABBREV=v['ABBREV'], DESCRIPTION=v['DESCRIPTION']) for v in self.properties_key.values()]
-        return tabulate.tabulate(key_dict, headers='keys', floatfmt='.2f', tablefmt=_table_fmt)
+    def schema_table(self) -> str:
+        """ :attr:`schema` formatted as a printable table string. """
+        headers = {key: key.upper() for key in list(self.schema.values())[0].keys()}
+        return tabulate.tabulate(self.schema.values(), headers=headers, floatfmt='.2f', tablefmt=_table_fmt)
+
+    @property
+    def refl_bands(self) -> Union[List[str], None]:
+        """ List of spectral / reflectance band names, if any. """
+        if not self._stac:
+            return None
+        return [bname for bname, bdict in self._stac.band_props.items() if 'center_wavelength' in bdict]
 
     def _get_properties(self, ee_collection: ee.ImageCollection) -> Dict:
         """ Retrieve properties of images in a given Earth Engine image collection. """
 
         # the properties to retrieve
-        prop_key_list = ee.List([item['PROPERTY'] for item in self.info['properties']])
+        prop_key_list = ee.List(list(self.schema.keys()))
 
         def aggregrate_props(ee_image, coll_list):
             im_dict = ee_image.toDictionary(prop_key_list)
@@ -258,36 +297,38 @@ class MaskedCollection:
             props_dict[prop_dict['system:id']] = prop_dict
         return props_dict
 
-    def _get_properties_table(self, properties: Dict, properties_key: Dict = None) -> str:
+    def _get_properties_table(self, properties: Dict, schema: Dict = None) -> str:
         """
-        Format the given properties into a table.  Orders properties (columns) according `properties_key` and replaces
-        long form property names with abbreviations.
+        Format the given properties into a table.  Orders properties (columns) according :attr:`schema` and
+        replaces long form property names with abbreviations.
         """
-        if not properties_key:
-            properties_key = self.properties_key
+        if not schema:
+            schema = self.schema
 
         abbrev_props = []
         for im_id, im_prop_dict in properties.items():
             abbrev_dict = OrderedDict()
-            for prop_name, key_dict in properties_key.items():
+            for prop_name, key_dict in schema.items():
                 if prop_name in im_prop_dict:
                     if prop_name == 'system:time_start':  # convert timestamp to date string
                         dt = datetime.utcfromtimestamp(im_prop_dict[prop_name] / 1000)
-                        abbrev_dict[key_dict['ABBREV']] = datetime.strftime(dt, '%Y-%m-%d %H:%M')
+                        abbrev_dict[key_dict['abbrev']] = datetime.strftime(dt, '%Y-%m-%d %H:%M')
                     else:
-                        abbrev_dict[key_dict['ABBREV']] = im_prop_dict[prop_name]
+                        abbrev_dict[key_dict['abbrev']] = im_prop_dict[prop_name]
             abbrev_props.append(abbrev_dict)
         return tabulate.tabulate(abbrev_props, headers='keys', floatfmt='.2f', tablefmt=_table_fmt)
 
     def _prepare_for_composite(
-        self, method=None, mask=True, resampling=BaseImage._default_resampling, date=None,
-        region=None, **kwargs
-    ):
+        self, method: CompositeMethod, mask: bool = True, resampling: Union[ResamplingMethod, str] = None,
+        date: str = None, region: Dict = None, **kwargs
+    ) -> ee.ImageCollection:
         """
-        Prepare the Earth Engine collection for compositing. See MaskedCollection.composite() for
+        Prepare the Earth Engine collection for compositing. See :meth:`~MaskedCollection.composite` for
         parameter descriptions.
         """
 
+        date = parse_date(date, 'date')
+
         if not self._filtered:
             raise UnfilteredError(
                 'Composites can only be created from collections returned by `search()` and `from_list()`'
@@ -296,24 +337,28 @@ class MaskedCollection:
         method = CompositeMethod(method)
         resampling = ResamplingMethod(resampling) if resampling else BaseImage._default_resampling
         if (method == CompositeMethod.q_mosaic) and (self.image_type == MaskedImage):
-            # TODO get a list of supported collections, report this in CLI help too
-            raise ValueError(f'The `q-mosaic` method is not supported for this ("{self.name}") collection.')
+            cloud_coll_str = ", ".join(schema.cloud_coll_names)
+            raise ValueError(
+                f'The `q-mosaic` method is not supported for this collection: {self.name}.  It is supported for the '
+                f'{cloud_coll_str} collections only.'
+            )
 
         def prepare_image(ee_image: ee.Image):
-            """ Prepare an EE image for use in compositing. """
+            """ Prepare an Earth Engine image for use in compositing. """
             if date and (method in [CompositeMethod.mosaic, CompositeMethod.q_mosaic]):
                 date_dist = ee.Number(ee_image.get('system:time_start')).subtract(ee.Date(date).millis()).abs()
                 ee_image = ee_image.set('DATE_DIST', date_dist)
 
+            # TODO: there is a design flaw here, if composite() is called >1x, but with different kwargs,
+            #  only the first kwargs are applied. similarly if composite is called on a searched collection,
+            #  only the search kwargs get applied.  this was done so that composite images could be wrapped in
+            #  MaskedImage w/o trying to find the masks again.  urgh.
             gd_image = self.image_type(ee_image, **kwargs)
             if region and (method in [CompositeMethod.mosaic, CompositeMethod.q_mosaic]):
-                gd_image.set_region_stats(region)
+                gd_image._set_region_stats(region=region, scale=self._stats_scale)
             if mask:
                 gd_image.mask_clouds()
-            if resampling != BaseImage._default_resampling:
-                return gd_image.ee_image.resample(resampling.value)
-            else:
-                return gd_image.ee_image
+            return resample(gd_image.ee_image, resampling)
 
         ee_collection = self._ee_collection.map(prepare_image)
 
@@ -332,39 +377,49 @@ class MaskedCollection:
 
         return ee_collection
 
-    def search(self, start_date, end_date, region, fill_portion=None, cloudless_portion=None, **kwargs):
+    def search(
+        self, start_date: Union[datetime, str], end_date: Union[datetime, str], region: dict,
+        fill_portion: float = None, cloudless_portion: float = None, **kwargs
+    ) -> 'MaskedCollection':
         """
-        Search for images based on date, region and cloudless portion criteria.
+        Search for images based on date, region and filled/cloudless portion criteria.
 
         Parameters
         ----------
-        start_date : datetime.datetime
-            Start date (UTC).
-        end_date : datetime.datetime, optional
-            End date (UTC). [default: start_date + 1 day]
+        start_date : datetime, str
+            Start date (UTC).  In '%Y-%m-%d' format if a string.
+        end_date : datetime, str, optional
+            End date (UTC).  In '%Y-%m-%d' format if a string.  If None, ``end_date`` is set to a day after
+            ``start_date``.
         region : dict, ee.Geometry
             Polygon in WGS84 specifying a region that images should intersect.
         fill_portion: float, optional
-            Minimum portion (%) of valid/filled image pixels.
+            Minimum portion (%) of filled (valid) image pixels.
         cloudless_portion: float, optional
             Minimum portion (%) of cloud/shadow free image pixels.
-        kwargs: optional
-            Cloud/shadow masking parameters - see MaskedImage.__init__() for details.
+        **kwargs
+            Optional cloud/shadow masking parameters - see :meth:`geedim.mask.MaskedImage.__init__` for details.
 
         Returns
         -------
-        gd_collection: MaskedCollection
-            A new MaskedCollection instance containing the search filtered images.
+        MaskedCollection
+            Filtered MaskedCollection instance containing the search result image(s).
         """
+        start_date = parse_date(start_date, 'start_date')
+
         if end_date is None:
+            # set end_date a day later than start_date
             end_date = start_date + timedelta(days=1)
+        else:
+            end_date = parse_date(end_date, 'end_date')
+
         if end_date <= start_date:
             raise ValueError('`end_date` must be at least a day later than `start_date`')
 
         def set_region_stats(ee_image: ee.Image):
             """ Find filled and cloud/shadow free portions inside the search region for a given image.  """
             gd_image = self.image_type(ee_image, **kwargs)
-            gd_image.set_region_stats(region)
+            gd_image._set_region_stats(region, scale=self._stats_scale)
             return gd_image.ee_image
 
         # filter the image collection, finding cloud/shadow masks and region stats
@@ -383,50 +438,41 @@ class MaskedCollection:
         return gd_collection
 
     def composite(
-        self, method=None, mask=True, resampling=BaseImage._default_resampling, date=None,
-        region=None, **kwargs
-    ):
+        self, method: Union[CompositeMethod, str] = None, mask: bool = True,
+        resampling: Union[ResamplingMethod, str] = None, date: Union[datetime, str] = None,
+        region: dict = None, **kwargs
+    ) -> MaskedImage:
         """
         Create a composite image from the encapsulated image collection.
 
         Parameters
         ----------
-        method: CompositeMethod, optional
-            Method for finding each composite pixel from the collection of corresponding input image pixels.  One of:
-                `q_mosaic`: Use the unmasked pixel with the highest quality (i.e. distance to nearest cloud). When more
-                    than one pixel shares the highest quality value, the first of the competing pixels is used. Valid
-                    for cloud/shadow maskable image collections only (Sentinel-2 TOA and SR, and Landsat4-9 level 2
-                    collection 2).
-                `mosaic`: Use the first unmasked pixel.
-                `medoid`: Use the medoid of the unmasked pixels.  The medoid selects the image pixel (across
-                    bands) from the image having the minimum summed diff (across bands) from the median of the
-                    collection images. Maintains the original relationship between bands.
-                    See https://www.mdpi.com/2072-4292/5/12/6481 for detail.
-                `median`: Use the median of the unmasked pixels.
-                `mode`: Use the mode of the unmasked pixels.
-                `mean`: use the mean of the unmasked pixels.
+        method: CompositeMethod, str, optional
+            Method for finding each composite pixel from the stack of corresponding input image pixels. See
+            :class:`~geedim.enums.CompositeMethod` for available options.  By default, `q-mosaic` is used for
+            cloud/shadow mask supported collections, `mosaic` otherwise.
         mask: bool, optional
-            Whether to apply the cloud/shadow mask, or fill (valid pixel) mask, in the case of images without
-            support for cloud/shadow masking.  [default: True].
-        resampling: ResamplingMethod, optional
-            The resampling method to use on collection images prior to compositing.  If 'near', no resampling is done
-            [default: 'near'].
-        date: datetime.datetime, optional
+            Whether to apply the cloud/shadow mask; or fill (valid pixel) mask, in the case of images without
+            support for cloud/shadow masking.
+        resampling: ResamplingMethod, str, optional
+            Resampling method to use on collection images prior to compositing.  If None, `near` resampling is used
+            (the default).  See :class:`~geedim.enums.ResamplingMethod` for available options.
+        date: datetime, str, optional
             Sort collection images by their absolute difference in capture time from this date.  Useful for
             prioritising pixels from images closest to this date.  Valid for the `q-mosaic`
-            and `mosaic` methods only.  If None, no time difference sorting is done. [default: None].
+            and `mosaic` ``method`` only.  If None, no time difference sorting is done (the default).
         region: dict, optional
-            Sort collection images by their cloudless portion inside this geojson polygon (only if `date` is not
+            Sort collection images by their cloudless portion inside this geojson polygon (only if ``date`` is not
             specified).  This is useful to prioritise pixels from the least cloudy image(s).  Valid for the `q-mosaic`
-            and `mosaic` methods.  If `date` and `region` are not specified, collection images are sorted by their
-            capture date.
-        kwargs: optional
-            Cloud/shadow masking parameters - see MaskedImage.__init__() for details.
+            and `mosaic` ``method`` only.  If None, no cloudless portion sorting is done (the default). If ``date`` and
+            ``region`` are not specified, collection images are sorted by their capture date.
+        **kwargs
+            Optional cloud/shadow masking parameters - see :meth:`geedim.mask.MaskedImage.__init__` for details.
 
         Returns
         -------
-        comp_image: MaskedImage
-            The composite image.
+        MaskedImage
+            Composite image.
         """
 
         if isinstance(date, str):
@@ -449,13 +495,10 @@ class MaskedCollection:
         elif method == CompositeMethod.mosaic:
             comp_image = ee_collection.mosaic()
         elif method == CompositeMethod.median:
-            # TODO: S2 median gives 'Output of image computation is too large' on download dtype=uint16
             comp_image = ee_collection.median()
         elif method == CompositeMethod.medoid:
             # limit medoid to surface reflectance bands
-            # TODO: we need another way to get sr_bands if we are removing collection_info
-            sr_bands = [band_dict['id'] for band_dict in self.info['bands']]
-            comp_image = medoid.medoid(ee_collection, bands=sr_bands)
+            comp_image = medoid.medoid(ee_collection, bands=self.refl_bands)
         elif method == CompositeMethod.mode:
             comp_image = ee_collection.mode()
         elif method == CompositeMethod.mean:
@@ -468,9 +511,10 @@ class MaskedCollection:
         if len(props) == 0:
             raise ValueError('The collection is empty.')
         props_str = self._get_properties_table(props)
-        comp_image = comp_image.set('COMPONENT_IMAGES', 'TABLE:\n' + props_str)
+        comp_image = comp_image.set('INPUT_IMAGES', 'TABLE:\n' + props_str)
 
         # construct an ID for the composite
+        # TODO: these dates are being shifted from capture dates, something with time zone is confused
         dates = [datetime.utcfromtimestamp(item['system:time_start'] / 1000) for item in props.values()]
         start_date = min(dates).strftime('%Y_%m_%d')
         end_date = max(dates).strftime('%Y_%m_%d')
@@ -483,9 +527,10 @@ class MaskedCollection:
         comp_image = comp_image.set('system:id', comp_id)  # sets root 'id' property
         comp_image = comp_image.set('system:index', comp_id)  # sets 'properties'->'system:index'
 
-        # set the composite capture time to the capture time of the first component image (sets
-        # 'properties'->'system:time_start')
-        comp_image = comp_image.set('system:time_start', min(dates).timestamp() * 1000)
+        # set the composite capture time to the capture time of the first input image.
+        # note that we must specify the timezone as utc, otherwise .timestamp() assumes local time.
+        timestamp = min(dates).replace(tzinfo=timezone.utc).timestamp() * 1000
+        comp_image = comp_image.set('system:time_start', timestamp)
         gd_comp_image = self.image_type(comp_image)
         gd_comp_image._id = comp_id  # avoid getInfo() for id property
         return gd_comp_image
